<?php

namespace Oro\Bundle\ApiBundle\Processor\Shared\JsonApi;

use Oro\Bundle\ApiBundle\Filter\IncludeFilter;
use Oro\Bundle\ApiBundle\Processor\Context;
use Oro\Bundle\ApiBundle\Request\DataType;
use Oro\Component\ChainProcessor\ContextInterface;
use Oro\Component\ChainProcessor\ProcessorInterface;

/**
 * Adds "include" filter that can be used to specify which related entities should be returned.
 * As this filter has influence on the entity configuration, it is handled by a separate processor.
 * @see \Oro\Bundle\ApiBundle\Processor\Shared\JsonApi\HandleIncludeFilter
 */
class AddIncludeFilter implements ProcessorInterface
{
    public const FILTER_KEY         = 'include';
    public const FILTER_DESCRIPTION =
        'A list of related entities to be included. Comma-separated paths, e.g. \'comments,comments.author\'.';

    /**
     * {@inheritdoc}
     */
    public function process(ContextInterface $context)
    {
        /** @var Context $context */

        $filters = $context->getFilters();
        if ($filters->has(self::FILTER_KEY)) {
            // the "include" filter is already added
            return;
        }

        $config = $context->getConfig();
        if (null === $config || !$config->isInclusionEnabled()) {
            // the "include" filter is disabled
            return;
        }

        $filter = new IncludeFilter(DataType::STRING, self::FILTER_DESCRIPTION);
        $filter->setArrayAllowed(true);
        $filters->add(self::FILTER_KEY, $filter);
    }
}
